@@ -21,9 +21,9 @@ import typing as ty
 import warnings
 
 from keystoneauth1 import access as ksa_access
+from keystoneauth1.identity import base as ksa_identity_base
 from keystoneauth1 import session as ksa_session
 from openstack.config import cloud_region
-from openstack.config import loader as config  # noqa
 from openstack import connection
 from oslo_utils import strutils
 
@@ -59,7 +59,7 @@ class ClientCache:
 
 
 class _PasswordHelper(ty.Protocol):
-    def __call__(self, prompt: ty.Optional[str] = None) -> str: ...
+    def __call__(self, prompt: str | None = None) -> str: ...
 
 
 class ClientManager:
@@ -75,10 +75,10 @@ class ClientManager:
     def __init__(
         self,
         cli_options: cloud_region.CloudRegion,
-        api_version: ty.Optional[ty.Dict[str, str]],
-        pw_func: ty.Optional[_PasswordHelper] = None,
-        app_name: ty.Optional[str] = None,
-        app_version: ty.Optional[str] = None,
+        api_version: dict[str, str],
+        pw_func: _PasswordHelper | None = None,
+        app_name: str | None = None,
+        app_version: str | None = None,
     ) -> None:
         """Set up a ClientManager
 
@@ -106,7 +106,7 @@ class ClientManager:
 
         self.timing = self._cli_options.timing
 
-        self._auth_ref = None
+        self._auth_ref: ksa_access.AccessInfo | None = None
 
         # self.verify is the Requests-compatible form
         # self.cacert is the form used by the legacy client libs
@@ -143,7 +143,6 @@ class ClientManager:
         This is deferred until authentication is actually attempted because
         it gets in the way of things that do not require auth.
         """
-
         if self._auth_setup_completed:
             return
 
@@ -152,8 +151,7 @@ class ClientManager:
 
         # Basic option checking to avoid unhelpful error messages
         auth.check_valid_authentication_options(
-            self._cli_options,
-            self.auth_plugin_name,
+            self._cli_options, self.auth_plugin_name
         )
 
         # Horrible hack alert...must handle prompt for null password if
@@ -172,7 +170,19 @@ class ClientManager:
         )
         self.auth = self._cli_options.get_auth()
 
+        if self.auth is None:
+            raise exceptions.AuthorizationFailure(
+                'Failed to load authentication information'
+            )
+
         if self._cli_options.service_provider:
+            if not isinstance(self.auth, ksa_identity_base.BaseIdentityPlugin):
+                raise exceptions.AuthorizationFailure(
+                    f'keystone-to-keystone (k2k) federation requires use '
+                    f'of an Identity-based authentication plugin. '
+                    f'Got: {self.auth_plugin_name}'
+                )
+
             self.auth = auth.get_keystone2keystone_auth(
                 self.auth,
                 self._cli_options.service_provider,
@@ -212,7 +222,7 @@ class ClientManager:
         )
 
     @property
-    def auth_ref(self) -> ty.Optional[ksa_access.AccessInfo]:
+    def auth_ref(self) -> ksa_access.AccessInfo | None:
         """Dereference will trigger an auth if it hasn't already"""
         if (
             not self._auth_required
@@ -223,14 +233,16 @@ class ClientManager:
         if not self._auth_ref:
             self.setup_auth()
             LOG.debug("Get auth_ref")
+            # narrow type: we know this is set now due to the setup_auth call
+            assert self.auth is not None
             self._auth_ref = self.auth.get_auth_ref(self.session)
         return self._auth_ref
 
-    def _override_for(self, service_type: str) -> ty.Optional[str]:
+    def _override_for(self, service_type: str) -> str | None:
         key = '{}_endpoint_override'.format(service_type.replace('-', '_'))
-        return ty.cast(ty.Optional[str], self._cli_options.config.get(key))
+        return ty.cast(str | None, self._cli_options.config.get(key))
 
-    def is_service_available(self, service_type: str) -> ty.Optional[bool]:
+    def is_service_available(self, service_type: str) -> bool | None:
         """Check if a service type is in the current Service Catalog"""
         # If there is an override, assume the service is available
         if self._override_for(service_type):
@@ -256,9 +268,9 @@ class ClientManager:
     def get_endpoint_for_service_type(
         self,
         service_type: str,
-        region_name: ty.Optional[str] = None,
+        region_name: str | None = None,
         interface: str = 'public',
-    ) -> ty.Optional[str]:
+    ) -> str | None:
         """Return the endpoint URL for the service type."""
         # Overrides take priority unconditionally
         override = self._override_for(service_type)
@@ -275,7 +287,7 @@ class ClientManager:
                 region_name=region_name,
                 interface=interface,
             )
-        else:
+        elif self.auth:
             # Get the passed endpoint directly from the auth plugin
             endpoint = self.auth.get_endpoint(
                 self.session,
@@ -283,5 +295,5 @@ class ClientManager:
             )
         return endpoint
 
-    def get_configuration(self) -> ty.Dict[str, ty.Any]:
+    def get_configuration(self) -> dict[str, ty.Any]:
         return copy.deepcopy(self._cli_options.config)
